@@ -11,7 +11,7 @@ our @EXPORT_OK = qw( download );
 use Term::ANSIScreen qw( :cursor :screen );
 use Term::Choose     qw( choose );
 
-use App::YTDL::Helper  qw( HIDE_CURSOR SHOW_CURSOR uni_system );
+use App::YTDL::Helper qw( HIDE_CURSOR SHOW_CURSOR uni_system );
 
 END { print SHOW_CURSOR }
 
@@ -38,7 +38,7 @@ sub _choose_fmt {
             push @format_ids, $fmt;
         }
     }
-    my @named_qualities = qw(best worst bestvideo worstvideo bestaudio worstaudio);
+    my @named_qualities = qw(best); # worst bestvideo worstvideo bestaudio worstaudio);
     push @choices, @named_qualities;
     push @format_ids, @named_qualities;
     my @fmt_str;
@@ -86,7 +86,7 @@ sub _choose_fmt {
 
 
 sub download {
-    my ( $set, $opt, $data, $chosen, $from_list ) = @_;
+    my ( $set, $opt, $data, $chosen ) = @_;
     if ( ! defined $chosen || ! keys %$chosen ) {
         return;
     }
@@ -123,16 +123,15 @@ sub download {
             my $ask_fmt_for_each_video = 0;
             if ( $qty eq 'manually' && @sorted_ids > 1 ) {
                 my ( $yes, $no ) = (  '- YES', '- NO' );
-                my $prompt = 'Keep format choices for all videos?';
+                my $prompt = "\n" . ( $data->{$ex}{$up}{$sorted_ids[0]}{uploader} // $up ) . ':' . "\n";
+                $prompt .= 'Keep format choices for chosen videos?';
                 # Choose
                 my $choice = choose(
                     [ undef, $yes, $no ],
-                    { prompt => $prompt, undef => 'EXIT', layout => 3 }
+                    { prompt => $prompt, undef => '<<', layout => 3 }
                 );
                 if ( ! defined $choice ) {
-                    print locate 1, 1;
-                    print cldown;
-                    exit;
+                    next UPLOADER_ID;
                 }
                 elsif ( $choice eq $no ) {
                     $ask_fmt_for_each_video = 1;
@@ -146,22 +145,35 @@ sub download {
             my $backup_fmt_str;
 
             VIDEO_ID: for my $id ( @sorted_ids ) {
+                my $count_of_total = $data->{$ex}{$up}{$id}{count} . '/' . $total;
+                if ( ! defined $data->{$ex}{$up}{$id}{fmt_to_info} ) {
+                    say $count_of_total;
+                    say '[' . $ex . '] ' . $id;
+                    say $data->{$ex}{$up}{$id}{title};
+                    say 'Skipped: ' . $set->{failed_fetching_data};
+                    next VIDEO_ID;
+                }
                 my @cmd = @{$set->{youtube_dl}};
                 #push @cmd, '--skip-download';
                 if ( $qty eq 'manually' ) {
                     if ( $ask_fmt_for_each_video || ! $fmt_str ) {
-                        my $count_of_total = $data->{$ex}{$up}{$id}{count} . '/' . $total;
                         my $title = "\n" . $count_of_total . ' "' . $data->{$ex}{$up}{$id}{title} . '"';
                         $fmt_str = _choose_fmt( $data, $ex, $up, $id, $title );
                         if ( ! defined $fmt_str ) {
-                            print $count_of_total . ' skipped' . "\n";
+                            say $count_of_total;
+                            say '[' . $ex . '] ' . $id;
+                            say $data->{$ex}{$up}{$id}{title};
+                            say 'Skipped: user request';
                             next VIDEO_ID;
                         }
                     }
-                    push @cmd, '-f', $fmt_str;
+                    if ( $fmt_str ne 'best' ) {
+                        push @cmd, '-f', $fmt_str;
+                    }
                 }
                 elsif ( $qty eq 'best' ) { #
                     # no format required, because 'best' is default
+                    #push @cmd, '-f', 'best';
                 }
                 else {
                     push @cmd, '-f', "bestvideo[height$qty]+bestaudio/best[height$qty]";
@@ -177,7 +189,7 @@ sub download {
 
                 push @cmd, '-o', $output;
                 push @cmd, '--', $data->{$ex}{$up}{$id}{webpage_url};
-                print $data->{$ex}{$up}{$id}{count} . '/' . $total . ' ';
+                print $data->{$ex}{$up}{$id}{count} . '/' . $total . "\n";
                 my $exit_value = uni_system( @cmd );
                 if ( $exit_value != 0 ) {
                     $count->{$id}++;
@@ -201,14 +213,8 @@ sub download {
             }
         }
     }
-    if ( $from_list && $total ) {
-        choose(
-            [ " " ],
-            { prompt => "Press Enter:" }
-        );
-    }
     print SHOW_CURSOR;
-    return;
+    return $total;
 }
 
 
